@@ -2,10 +2,11 @@
 
 bufaux.c
 
-Author: Tatu Ylonen <ylo@cs.hut.fi>
+Author: Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
-                   All rights reserved
+Copyright (c) 1995 Tatu Ylonen <ylo@ssh.fi>, Espoo, Finland
+Copyright (c) 1995-1999 SSH Communications Security Oy, Espoo, Finland
+                        All rights reserved
 
 Created: Wed Mar 29 02:24:47 1995 ylo
 
@@ -15,17 +16,20 @@ Buffers.
 */
 
 /*
- * $Id: bufaux.c,v 1.1.1.1 1996/02/18 21:38:11 ylo Exp $
+ * $Id: bufaux.c,v 1.2 1999/11/17 17:04:40 tri Exp $
  * $Log: bufaux.c,v $
- * Revision 1.1.1.1  1996/02/18  21:38:11  ylo
- * 	Imported ssh-1.2.13.
+ * Revision 1.2  1999/11/17 17:04:40  tri
+ * 	Fixed copyright notices.
+ *
+ * Revision 1.1.1.1  1996/02/18 21:38:11  ylo
+ *      Imported ssh-1.2.13.
  *
  * Revision 1.3  1995/08/29  22:19:52  ylo
- * 	Removed extra '&'.
+ *      Removed extra '&'.
  *
  * Revision 1.2  1995/07/13  01:17:49  ylo
- * 	Removed "Last modified" header.
- * 	Added cvs log.
+ *      Removed "Last modified" header.
+ *      Added cvs log.
  *
  * $Endlog$
  */
