@@ -2,10 +2,11 @@
 
 buffer.c
 
-Author: Tatu Ylonen <ylo@cs.hut.fi>
+Author: Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
-                   All rights reserved
+Copyright (c) 1995 Tatu Ylonen <ylo@ssh.fi>, Espoo, Finland
+Copyright (c) 1995-1999 SSH Communications Security Oy, Espoo, Finland
+                        All rights reserved
 
 Created: Sat Mar 18 04:15:33 1995 ylo
 
@@ -14,14 +15,17 @@ Functions for manipulating fifo buffers (that can grow if needed).
 */
 
 /*
- * $Id: buffer.c,v 1.1.1.1 1996/02/18 21:38:11 ylo Exp $
+ * $Id: buffer.c,v 1.2 1999/11/17 17:04:40 tri Exp $
  * $Log: buffer.c,v $
- * Revision 1.1.1.1  1996/02/18  21:38:11  ylo
- * 	Imported ssh-1.2.13.
+ * Revision 1.2  1999/11/17 17:04:40  tri
+ * 	Fixed copyright notices.
+ *
+ * Revision 1.1.1.1  1996/02/18 21:38:11  ylo
+ *      Imported ssh-1.2.13.
  *
  * Revision 1.2  1995/07/13  01:18:46  ylo
- * 	Removed "Last modified" header.
- * 	Added cvs log.
+ *      Removed "Last modified" header.
+ *      Added cvs log.
  *
  * $Endlog$
  */
@@ -94,7 +98,7 @@ void buffer_append_space(Buffer *buffer, char **datap, unsigned int len)
   if (buffer->offset > buffer->alloc / 2)
     {
       memmove(buffer->buf, buffer->buf + buffer->offset,
-	      buffer->end - buffer->offset);
+              buffer->end - buffer->offset);
       buffer->end -= buffer->offset;
       buffer->offset = 0;
       goto restart;
