@@ -2,10 +2,11 @@
 
 userfile.c
 
-Author: Tatu Ylonen <ylo@cs.hut.fi>
+Author: Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
-                   All rights reserved
+Copyright (c) 1995 Tatu Ylonen <ylo@ssh.fi>, Espoo, Finland
+Copyright (c) 1995-1999 SSH Communications Security Oy, Espoo, Finland
+                        All rights reserved
 
 Created: Wed Jan 24 20:19:53 1996 ylo
 
@@ -13,68 +14,76 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 
 /*
  * $Log: userfile.c,v $
- * Revision 1.18  1998/05/23  20:28:26  kivinen
- * 	Changed () -> (void).
+ * Revision 1.20  1999/11/17 17:05:01  tri
+ * 	Fixed copyright notices.
+ *
+ * Revision 1.19  1999/02/21 19:53:01  ylo
+ *      Intermediate commit of ssh1.2.27 stuff.
+ *      Main change is sprintf -> snprintf; however, there are also
+ *      many other changes.
+ *
+ * Revision 1.18  1998/05/23 20:28:26  kivinen
+ *      Changed () -> (void).
  *
  * Revision 1.17  1998/04/17  00:43:08  kivinen
- * 	Freebsd login capabilities support.
+ *      Freebsd login capabilities support.
  *
  * Revision 1.16  1997/05/13 22:30:05  kivinen
- * 	Added some casts.
+ *      Added some casts.
  *
  * Revision 1.15  1997/03/26 07:10:04  kivinen
- * 	Changed uid 0 to UID_ROOT.
+ *      Changed uid 0 to UID_ROOT.
  *
  * Revision 1.14  1997/03/25 05:47:00  kivinen
- * 	Changed USERFILE_GET_DES_1_MAGIC_PHRASE to call
- * 	userfile_get_des_1_magic_phrase.
+ *      Changed USERFILE_GET_DES_1_MAGIC_PHRASE to call
+ *      userfile_get_des_1_magic_phrase.
  *
  * Revision 1.13  1997/03/19 17:53:46  kivinen
- * 	Added USERFILE_GET_DES_1_MAGIC_PHRASE.
+ *      Added USERFILE_GET_DES_1_MAGIC_PHRASE.
  *
  * Revision 1.12  1996/11/27 14:30:07  ttsalo
  *     Added group-writeability #ifdef
  *
  * Revision 1.11  1996/10/29 22:48:23  kivinen
- * 	Removed USERFILE_LOCAL_SOCK and USERFILE_SEND.
+ *      Removed USERFILE_LOCAL_SOCK and USERFILE_SEND.
  *
  * Revision 1.10  1996/10/07 11:40:20  ttsalo
- * 	Configuring for hurd and a small fix to do_popen()
- * 	from "Charles M. Hannum" <mycroft@gnu.ai.mit.edu> added.
+ *      Configuring for hurd and a small fix to do_popen()
+ *      from "Charles M. Hannum" <mycroft@gnu.ai.mit.edu> added.
  *
  * Revision 1.9  1996/10/04 01:01:49  kivinen
- * 	Added printing of path to fatal calls in userfile_open and and
- * 	userfile_local_socket_connect. Fixed userfile_open to
- * 	userfile_local_socket_connect in calls to fatal in
- * 	userfile_local_socket_connect.
+ *      Added printing of path to fatal calls in userfile_open and and
+ *      userfile_local_socket_connect. Fixed userfile_open to
+ *      userfile_local_socket_connect in calls to fatal in
+ *      userfile_local_socket_connect.
  *
  * Revision 1.8  1996/09/27 17:18:06  ylo
- * 	Fixed a typo.
+ *      Fixed a typo.
  *
  * Revision 1.7  1996/09/08 17:21:08  ttsalo
- * 	A lot of changes in agent-socket handling
+ *      A lot of changes in agent-socket handling
  *
  * Revision 1.6  1996/09/04 12:39:59  ttsalo
- * 	Added connecting to unix-domain socket
+ *      Added connecting to unix-domain socket
  *
  * Revision 1.5  1996/08/13 09:04:19  ttsalo
- * 	Home directory, .ssh and .ssh/authorized_keys are now
- * 	checked for wrong owner and group & world writeability.
+ *      Home directory, .ssh and .ssh/authorized_keys are now
+ *      checked for wrong owner and group & world writeability.
  *
  * Revision 1.4  1996/05/29 07:37:31  ylo
- * 	Do setgid and initgroups when initializing to read as user.
+ *      Do setgid and initgroups when initializing to read as user.
  *
  * Revision 1.3  1996/04/26 18:10:45  ylo
- * 	Wrong file descriptors were closed in the forked child in
- * 	do_popen.  This caused dup2 to fail on some machines, which in
- * 	turn resulted in X11 authentication failing on some machines.
+ *      Wrong file descriptors were closed in the forked child in
+ *      do_popen.  This caused dup2 to fail on some machines, which in
+ *      turn resulted in X11 authentication failing on some machines.
  *
  * Revision 1.2  1996/02/18 21:48:46  ylo
- * 	Close pipes after popen fork.
- * 	New function userfile_close_pipes.
+ *      Close pipes after popen fork.
+ *      New function userfile_close_pipes.
  *
  * Revision 1.1.1.1  1996/02/18 21:38:11  ylo
- * 	Imported ssh-1.2.13.
+ *      Imported ssh-1.2.13.
  *
  * $EndLog$
  */
@@ -85,81 +94,81 @@ Created: Wed Jan 24 20:19:53 1996 ylo
       type but not length itself), followed by one byte containing
       the packet type.
 
-	1 USERFILE_OPEN
-          string 	file name
-	  int32 	flags
-	  int32		mode
+        1 USERFILE_OPEN
+          string        file name
+          int32         flags
+          int32         mode
 
-	2 USERFILE_OPEN_REPLY
-	  int32		handle (-1 if error)
+        2 USERFILE_OPEN_REPLY
+          int32         handle (-1 if error)
 
-	3 USERFILE_READ
-	  int32		handle
-	  int32		max_bytes
+        3 USERFILE_READ
+          int32         handle
+          int32         max_bytes
 
         4 USERFILE_READ_REPLY
-	  string	data	  ;; empty data means EOF
+          string        data      ;; empty data means EOF
 
-	5 USERFILE_WRITE
-	  int32		handle
-	  string	data
+        5 USERFILE_WRITE
+          int32         handle
+          string        data
 
-	6 USERFILE_WRITE_REPLY
-	  int32		bytes_written  ;; != length of data means error
-	
-	7 USERFILE_CLOSE
-	  int32		handle
+        6 USERFILE_WRITE_REPLY
+          int32         bytes_written  ;; != length of data means error
+        
+        7 USERFILE_CLOSE
+          int32         handle
 
-	8 USERFILE_CLOSE_REPLY
-	  int32		return value
+        8 USERFILE_CLOSE_REPLY
+          int32         return value
 
-	9 USERFILE_LSEEK
-	  int32		handle
-	  int32		offset
-	  int32		whence
+        9 USERFILE_LSEEK
+          int32         handle
+          int32         offset
+          int32         whence
 
        10 USERFILE_LSEEK_REPLY
-	  int32 	returned_offset
+          int32         returned_offset
 
        11 USERFILE_MKDIR
-	  string	path
-	  int32		mode
+          string        path
+          int32         mode
 
        12 USERFILE_MKDIR_REPLY
-	  int32		return value
+          int32         return value
 
        13 USERFILE_STAT
-	  string	path
+          string        path
 
        14 USERFILE_STAT_REPLY
-          int32		return value
-	  sizeof(struct stat) binary bytes (in host order and layout)
+          int32         return value
+          sizeof(struct stat) binary bytes (in host order and layout)
 
        15 USERFILE_REMOVE
-	  string	path
+          string        path
 
        16 USERFILE_REMOVE_REPLY
-	  int32		return value
+          int32         return value
 
        17 USERFILE_POPEN
-	  string	command
-	  string	type
+          string        command
+          string        type
 
        18 USERFILE_POPEN_REPLY
-	  int32		handle (-1 if error)
+          int32         handle (-1 if error)
 
        19 USERFILE_PCLOSE
-          int32		handle
+          int32         handle
 
        20 USERFILE_PCLOSE_REPLY
-	  int32		return value
+          int32         return value
 
        21 USERFILE_GET_DES_1_MAGIC_PHRASE
 
        22 USERFILE_GET_DES_1_MAGIC_PHRASE_REPLY
-	  string	data
-	  
-	  */
+          string        data
+          
+          */
 
 #include "includes.h"
 #include <gmp.h>
@@ -180,27 +189,27 @@ Created: Wed Jan 24 20:19:53 1996 ylo
 #endif
 
 /* Protocol message types. */
-#define USERFILE_OPEN		1
-#define USERFILE_OPEN_REPLY	2
-#define USERFILE_READ		3
-#define USERFILE_READ_REPLY	4
-#define USERFILE_WRITE		5
-#define USERFILE_WRITE_REPLY	6
-#define USERFILE_CLOSE		7
-#define USERFILE_CLOSE_REPLY	8
-#define USERFILE_LSEEK		9
+#define USERFILE_OPEN           1
+#define USERFILE_OPEN_REPLY     2
+#define USERFILE_READ           3
+#define USERFILE_READ_REPLY     4
+#define USERFILE_WRITE          5
+#define USERFILE_WRITE_REPLY    6
+#define USERFILE_CLOSE          7
+#define USERFILE_CLOSE_REPLY    8
+#define USERFILE_LSEEK          9
 #define USERFILE_LSEEK_REPLY   10
-#define USERFILE_MKDIR	       11
+#define USERFILE_MKDIR         11
 #define USERFILE_MKDIR_REPLY   12
-#define USERFILE_STAT	       13
+#define USERFILE_STAT          13
 #define USERFILE_STAT_REPLY    14
-#define USERFILE_REMOVE	       15
+#define USERFILE_REMOVE        15
 #define USERFILE_REMOVE_REPLY  16
-#define USERFILE_POPEN	       17
+#define USERFILE_POPEN         17
 #define USERFILE_POPEN_REPLY   18
-#define USERFILE_PCLOSE	       19
+#define USERFILE_PCLOSE        19
 #define USERFILE_PCLOSE_REPLY  20
-#define USERFILE_GET_DES_1_MAGIC_PHRASE	       21
+#define USERFILE_GET_DES_1_MAGIC_PHRASE        21
 #define USERFILE_GET_DES_1_MAGIC_PHRASE_REPLY  22
 
 
@@ -254,16 +263,16 @@ static void userfile_packet_send(void)
     {
       bytes = write(userfile_output, lenbuf + offset, len - offset);
       if (bytes <= 0)
-	fatal("userfile_packet_send: child has died: %s", strerror(errno));
+        fatal("userfile_packet_send: child has died: %.100s", strerror(errno));
     }
   
   len = buffer_len(&packet);
   for (offset = 0; offset < len; offset += bytes)
     {
       bytes = write(userfile_output, buffer_ptr(&packet) + offset, 
-		    len - offset);
+                    len - offset);
       if (bytes <= 0)
-	fatal("userfile_packet_send: child has died: %s", strerror(errno));
+        fatal("userfile_packet_send: child has died: %.100s", strerror(errno));
     }
 }
 
@@ -286,11 +295,11 @@ static int userfile_read_raw(void)
     {
       bytes = read(userfile_input, buf + offset, len - offset);
       if (bytes <= 0)
-	{
-	  if (getuid() == geteuid()) /* presumably child - be quiet */
-	    exit(0);
-	  fatal("userfile_read_raw: child has died: %s", strerror(errno));
-	}
+        {
+          if (getuid() == geteuid()) /* presumably child - be quiet */
+            exit(0);
+          fatal("userfile_read_raw: child has died: %.100s", strerror(errno));
+        }
     }
 
   len = GET_32BIT(buf);
@@ -302,10 +311,10 @@ static int userfile_read_raw(void)
     {
       bytes = len - offset;
       if (bytes > sizeof(buf))
-	bytes = sizeof(buf);
+        bytes = sizeof(buf);
       bytes = read(userfile_input, buf, bytes);
       if (bytes <= 0)
-	fatal("userfile_read_raw: child has died: %s", strerror(errno));
+        fatal("userfile_read_raw: child has died: %.100s", strerror(errno));
       buffer_append(&packet, (char *) buf, bytes);
     }
   return buffer_get_char(&packet);
@@ -320,7 +329,7 @@ static void userfile_packet_read(int expected_type)
   type = userfile_read_raw();
   if (type != expected_type)
     fatal("userfile_read_packet: unexpected packet type: got %d, expected %d",
-	  type, expected_type);
+          type, expected_type);
 }
 
 /* Forks and execs the given command.  Returns a file descriptor for
@@ -337,31 +346,31 @@ int do_popen(const char *command, const char *type)
   extern char **environ;
   
   if (pipe(fds) < 0)
-    fatal("pipe: %s", strerror(errno));
+    fatal("pipe: %.100s", strerror(errno));
   
   pid = fork();
   if (pid < 0)
-    fatal("fork: %s", strerror(errno));
+    fatal("fork: %.100s", strerror(errno));
   
   if (pid == 0)
     { /* Child */
 
       /* Close pipes to the parent; we do not wish to disclose them to a
-	 random user program. */
+         random user program. */
       close(userfile_fromparent);
       close(userfile_toparent);
 
       /* Set up file descriptors. */
       if (type[0] == 'r')
-	{
-	  if (dup2(fds[1], 1) < 0)
-	    perror("dup2 1");
-	}
+        {
+          if (dup2(fds[1], 1) < 0)
+            perror("dup2 1");
+        }
       else
-	{
-	  if (dup2(fds[0], 0) < 0)
-	    perror("dup2 0");
-	}
+        {
+          if (dup2(fds[0], 0) < 0)
+            perror("dup2 0");
+        }
       close(fds[0]);
       close(fds[1]);
 
@@ -375,24 +384,24 @@ int do_popen(const char *command, const char *type)
       /* Prune environment to remove any potentially dangerous variables. */
       i = 0;
       for (j = 0; environ[j] && i < sizeof(env)/sizeof(env[0]) - 1; j++)
-	if (strncmp(environ[j], "HOME=", 5) == 0 ||
-	    strncmp(environ[j], "USER=", 5) == 0 ||
-	    strncmp(environ[j], "HOME=", 5) == 0 ||
-	    strncmp(environ[j], "PATH=", 5) == 0 ||
-	    strncmp(environ[j], "LOGNAME=", 8) == 0 ||
-	    strncmp(environ[j], "TZ=", 3) == 0 ||
-	    strncmp(environ[j], "MAIL=", 5) == 0 ||
-	    strncmp(environ[j], "SHELL=", 6) == 0 ||
-	    strncmp(environ[j], "TERM=", 5) == 0 ||
-	    strncmp(environ[j], "DISPLAY=", 8) == 0 ||
-	    strncmp(environ[j], "PRINTER=", 8) == 0 ||
-	    strncmp(environ[j], "XAUTHORITY=", 11) == 0 ||
-	    strncmp(environ[j], "TERMCAP=", 8) == 0)
-	  env[i++] = environ[j];
+        if (strncmp(environ[j], "HOME=", 5) == 0 ||
+            strncmp(environ[j], "USER=", 5) == 0 ||
+            strncmp(environ[j], "HOME=", 5) == 0 ||
+            strncmp(environ[j], "PATH=", 5) == 0 ||
+            strncmp(environ[j], "LOGNAME=", 8) == 0 ||
+            strncmp(environ[j], "TZ=", 3) == 0 ||
+            strncmp(environ[j], "MAIL=", 5) == 0 ||
+            strncmp(environ[j], "SHELL=", 6) == 0 ||
+            strncmp(environ[j], "TERM=", 5) == 0 ||
+            strncmp(environ[j], "DISPLAY=", 8) == 0 ||
+            strncmp(environ[j], "PRINTER=", 8) == 0 ||
+            strncmp(environ[j], "XAUTHORITY=", 11) == 0 ||
+            strncmp(environ[j], "TERMCAP=", 8) == 0)
+          env[i++] = environ[j];
       env[i] = NULL;
 
       execve("/bin/sh", args, env);
-      fatal("execv /bin/sh failed: %s", strerror(errno));
+      fatal("execv /bin/sh failed: %.100s", strerror(errno));
     }
 
   /* Parent. */
@@ -424,161 +433,161 @@ static void userfile_child_server(void)
     {
       type = userfile_read_raw();
       switch (type)
-	{
-	case USERFILE_OPEN:
-	  path = buffer_get_string(&packet, NULL);
-	  flags = buffer_get_int(&packet);
-	  mode = buffer_get_int(&packet);
+        {
+        case USERFILE_OPEN:
+          path = buffer_get_string(&packet, NULL);
+          flags = buffer_get_int(&packet);
+          mode = buffer_get_int(&packet);
 
-	  ret = open(path, flags, mode);
+          ret = open(path, flags, mode);
 
-	  userfile_packet_start(USERFILE_OPEN_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_OPEN_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  xfree(path);
-	  break;
+          xfree(path);
+          break;
 
-	case USERFILE_READ:
-	  handle = buffer_get_int(&packet);
-	  max_bytes = buffer_get_int(&packet);
+        case USERFILE_READ:
+          handle = buffer_get_int(&packet);
+          max_bytes = buffer_get_int(&packet);
 
-	  if (max_bytes >= sizeof(buf))
-	    max_bytes = sizeof(buf);
-	  ret = read(handle, buf, max_bytes);
-	  if (ret < 0)
-	    ret = 0;
+          if (max_bytes >= sizeof(buf))
+            max_bytes = sizeof(buf);
+          ret = read(handle, buf, max_bytes);
+          if (ret < 0)
+            ret = 0;
 
-	  userfile_packet_start(USERFILE_READ_REPLY);
-	  buffer_put_string(&packet, buf, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_READ_REPLY);
+          buffer_put_string(&packet, buf, ret);
+          userfile_packet_send();
 
-	  break;
-	  
-	case USERFILE_WRITE:
-	  handle = buffer_get_int(&packet);
-	  cp = buffer_get_string(&packet, &len);
+          break;
+          
+        case USERFILE_WRITE:
+          handle = buffer_get_int(&packet);
+          cp = buffer_get_string(&packet, &len);
 
-	  ret = write(handle, cp, len);
+          ret = write(handle, cp, len);
 
-	  userfile_packet_start(USERFILE_WRITE_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_WRITE_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  xfree(cp);
-	  break;
+          xfree(cp);
+          break;
 
-	case USERFILE_CLOSE:
-	  handle = buffer_get_int(&packet);
+        case USERFILE_CLOSE:
+          handle = buffer_get_int(&packet);
 
-	  ret = close(handle);
+          ret = close(handle);
 
-	  userfile_packet_start(USERFILE_CLOSE_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_CLOSE_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  break;
+          break;
 
-	case USERFILE_LSEEK:
-	  handle = buffer_get_int(&packet);
-	  offset = buffer_get_int(&packet);
-	  whence = buffer_get_int(&packet);
+        case USERFILE_LSEEK:
+          handle = buffer_get_int(&packet);
+          offset = buffer_get_int(&packet);
+          whence = buffer_get_int(&packet);
 
-	  ret = lseek(handle, offset, whence);
+          ret = lseek(handle, offset, whence);
 
-	  userfile_packet_start(USERFILE_LSEEK_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_LSEEK_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  break;
+          break;
 
-	case USERFILE_MKDIR:
-	  path = buffer_get_string(&packet, NULL);
-	  mode = buffer_get_int(&packet);
+        case USERFILE_MKDIR:
+          path = buffer_get_string(&packet, NULL);
+          mode = buffer_get_int(&packet);
 
-	  ret = mkdir(path, mode);
+          ret = mkdir(path, mode);
 
-	  userfile_packet_start(USERFILE_MKDIR_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_MKDIR_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  xfree(path);
-	  break;
+          xfree(path);
+          break;
 
-	case USERFILE_STAT:
-	  path = buffer_get_string(&packet, NULL);
+        case USERFILE_STAT:
+          path = buffer_get_string(&packet, NULL);
 
-	  ret = stat(path, &st);
+          ret = stat(path, &st);
 
-	  userfile_packet_start(USERFILE_STAT_REPLY);
-	  buffer_put_int(&packet, ret);
-	  buffer_append(&packet, (void *)&st, sizeof(st));
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_STAT_REPLY);
+          buffer_put_int(&packet, ret);
+          buffer_append(&packet, (void *)&st, sizeof(st));
+          userfile_packet_send();
 
-	  xfree(path);
-	  break;
-	  
-	case USERFILE_REMOVE:
-	  path = buffer_get_string(&packet, NULL);
+          xfree(path);
+          break;
+          
+        case USERFILE_REMOVE:
+          path = buffer_get_string(&packet, NULL);
 
-	  ret = remove(path);
+          ret = remove(path);
 
-	  userfile_packet_start(USERFILE_REMOVE_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_REMOVE_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  xfree(path);
-	  break;
+          xfree(path);
+          break;
 
-	case USERFILE_POPEN:
-	  command = buffer_get_string(&packet, NULL);
-	  cp = buffer_get_string(&packet, NULL);
+        case USERFILE_POPEN:
+          command = buffer_get_string(&packet, NULL);
+          cp = buffer_get_string(&packet, NULL);
 
-	  ret = do_popen(command, cp);
+          ret = do_popen(command, cp);
 
-	  userfile_packet_start(USERFILE_POPEN_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_POPEN_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  xfree(command);
-	  xfree(cp);
-	  break;
+          xfree(command);
+          xfree(cp);
+          break;
 
-	case USERFILE_PCLOSE:
-	  handle = buffer_get_int(&packet);
+        case USERFILE_PCLOSE:
+          handle = buffer_get_int(&packet);
 
-	  ret = close(handle);
-	  ret2 = wait(NULL);
-	  if (ret >= 0)
-	    ret = ret2;
+          ret = close(handle);
+          ret2 = wait(NULL);
+          if (ret >= 0)
+            ret = ret2;
 
-	  userfile_packet_start(USERFILE_PCLOSE_REPLY);
-	  buffer_put_int(&packet, ret);
-	  userfile_packet_send();
+          userfile_packet_start(USERFILE_PCLOSE_REPLY);
+          buffer_put_int(&packet, ret);
+          userfile_packet_send();
 
-	  break;
+          break;
 
-	case USERFILE_GET_DES_1_MAGIC_PHRASE:
-	  {
-	    char *buf = NULL;
+        case USERFILE_GET_DES_1_MAGIC_PHRASE:
+          {
+            char *buf = NULL;
 #ifdef SECURE_RPC
-	    buf = userfile_get_des_1_magic_phrase(geteuid());
+            buf = userfile_get_des_1_magic_phrase(geteuid());
 #endif
-	    userfile_packet_start(USERFILE_GET_DES_1_MAGIC_PHRASE_REPLY);
-	    if (buf == NULL)
-	      buffer_put_string(&packet, "", 0);
-	    else
-	      {
-		buffer_put_string(&packet, buf, strlen(buf));
-		memset(buf, 0, strlen(buf));
-	      }
-	    userfile_packet_send();
-	  }
-	  break;
+            userfile_packet_start(USERFILE_GET_DES_1_MAGIC_PHRASE_REPLY);
+            if (buf == NULL)
+              buffer_put_string(&packet, "", 0);
+            else
+              {
+                buffer_put_string(&packet, buf, strlen(buf));
+                memset(buf, 0, strlen(buf));
+              }
+            userfile_packet_send();
+          }
+          break;
 
-	default:
-	  fatal("userfile_child_server: packet type %d", type);
-	}
+        default:
+          fatal("userfile_child_server: packet type %d", type);
+        }
     }
 }
 
@@ -589,7 +598,7 @@ static void userfile_child_server(void)
    user's uid.  The callback may be NULL. */
 
 void userfile_init(const char *username, uid_t uid, gid_t gid,
-		   void (*cleanup_callback)(void *), void *context)
+                   void (*cleanup_callback)(void *), void *context)
 {
   int fds[2], pid;
 
@@ -600,18 +609,18 @@ void userfile_init(const char *username, uid_t uid, gid_t gid,
   userfile_initialized = 1;
 
   if (pipe(fds) < 0)
-    fatal("pipe: %s", strerror(errno));
+    fatal("pipe: %.100s", strerror(errno));
   userfile_tochild = fds[1];
   userfile_fromparent = fds[0];
   
   if (pipe(fds) < 0)
-    fatal("pipe: %s", strerror(errno));
+    fatal("pipe: %.100s", strerror(errno));
   userfile_fromchild = fds[0];
   userfile_toparent = fds[1];
   
   pid = fork();
   if (pid < 0)
-    fatal("fork: %s", strerror(errno));
+    fatal("fork: %.100s", strerror(errno));
 
   if (pid != 0)
     { 
@@ -643,20 +652,20 @@ void userfile_init(const char *username, uid_t uid, gid_t gid,
       struct passwd * pw = getpwuid(uid);
       login_cap_t * lc = login_getuserclass(pw);
       if (setusercontext(lc, pw, uid,
-			 LOGIN_SETALL & ~(LOGIN_SETLOGIN | LOGIN_SETPATH |
-					  LOGIN_SETENV)) < 0)
-	fatal("setusercontext: %s", strerror(errno));
+                         LOGIN_SETALL & ~(LOGIN_SETLOGIN | LOGIN_SETPATH |
+                                          LOGIN_SETENV)) < 0)
+        fatal("setusercontext: %.100s", strerror(errno));
 #else
       if (setgid(gid) < 0)
-	fatal("setgid: %s", strerror(errno));
+        fatal("setgid: %.100s", strerror(errno));
 
 #ifdef HAVE_INITGROUPS
       if (initgroups((char *) username, gid) < 0)
-	fatal("initgroups: %s", strerror(errno));
+        fatal("initgroups: %.100s", strerror(errno));
 #endif /* HAVE_INITGROUPS */
 
       if (setuid(uid) < 0)
-	fatal("setuid: %s", strerror(errno));
+        fatal("setuid: %.100s", strerror(errno));
 #endif /* HAVE_LOGIN_CAP_H */
     }
 
@@ -736,17 +745,17 @@ UserFile userfile_open(uid_t uid, const char *path, int flags, mode_t mode)
     {
       handle = open(path, flags, mode);
       if (handle < 0)
-	return NULL;
+        return NULL;
       return userfile_make_handle(USERFILE_LOCAL, handle);
     }
 
   if (!userfile_initialized)
     fatal("userfile_open: using non-current uid but not initialized (uid=%d, path=%.50s)",
-	  (int)uid, path);
+          (int)uid, path);
   
   if (uid != userfile_uid)
     fatal("userfile_open: uid not current and not that of child: uid=%d, path=%.50s",
-	  (int)uid, path);
+          (int)uid, path);
 
   userfile_packet_start(USERFILE_OPEN);
   buffer_put_string(&packet, path, strlen(path));
@@ -810,7 +819,7 @@ static int userfile_fill(UserFile uf)
     case USERFILE_LOCAL:
       ret = read(uf->handle, uf->buf, sizeof(uf->buf));
       if (ret <= 0)
-	return 0;
+        return 0;
       uf->buf_first = 0;
       uf->buf_last = ret;
       break;
@@ -824,11 +833,11 @@ static int userfile_fill(UserFile uf)
       userfile_packet_read(USERFILE_READ_REPLY);
       cp = buffer_get_string(&packet, &len);
       if (len > sizeof(uf->buf))
-	fatal("userfile_fill: got more than data than requested");
+        fatal("userfile_fill: got more than data than requested");
       memcpy(uf->buf, cp, len);
       xfree(cp);
       if (len == 0)
-	return 0;
+        return 0;
       uf->buf_first = 0;
       uf->buf_last = len;
       break;
@@ -848,10 +857,10 @@ int userfile_getc(UserFile uf)
   if (uf->buf_first >= uf->buf_last)
     {
       if (!userfile_fill(uf))
-	return -1;
+        return -1;
       
       if (uf->buf_first >= uf->buf_last)
-	fatal("userfile_getc/fill error");
+        fatal("userfile_getc/fill error");
     }
   
   return uf->buf[uf->buf_first++];
@@ -872,7 +881,7 @@ int userfile_read(UserFile uf, void *buf, unsigned int len)
     {
       ch = userfile_getc(uf);
       if (ch == -1)
-	break;
+        break;
       ucp[i] = ch;
     }
   
@@ -896,24 +905,24 @@ int userfile_write(UserFile uf, const void *buf, unsigned int len)
     case USERFILE_REMOTE:
       ucp = buf;
       for (offset = 0; offset < len; )
-	{
-	  chunk_len = len - offset;
-	  if (chunk_len > 16000)
-	    chunk_len = 16000;
-	  
-	  userfile_packet_start(USERFILE_WRITE);
-	  buffer_put_int(&packet, uf->handle);
-	  buffer_put_string(&packet, ucp + offset, chunk_len);
-	  userfile_packet_send();
-	  
-	  userfile_packet_read(USERFILE_WRITE_REPLY);
-	  ret = buffer_get_int(&packet);
-	  if (ret < 0)
-	    return -1;
-	  offset += ret;
-	  if (ret != chunk_len)
-	    break;
-	}
+        {
+          chunk_len = len - offset;
+          if (chunk_len > 16000)
+            chunk_len = 16000;
+          
+          userfile_packet_start(USERFILE_WRITE);
+          buffer_put_int(&packet, uf->handle);
+          buffer_put_string(&packet, ucp + offset, chunk_len);
+          userfile_packet_send();
+          
+          userfile_packet_read(USERFILE_WRITE_REPLY);
+          ret = buffer_get_int(&packet);
+          if (ret < 0)
+            return -1;
+          offset += ret;
+          if (ret != chunk_len)
+            break;
+        }
       return offset;
 
     default:
@@ -940,10 +949,10 @@ char *userfile_gets(char *buf, unsigned int size, UserFile uf)
     {
       ch = userfile_getc(uf);
       if (ch == -1)
-	break;
+        break;
       buf[i++] = ch;
       if (ch == '\n')
-	break;
+        break;
     }
   if (i == 0)
     return NULL;
@@ -1063,17 +1072,17 @@ UserFile userfile_popen(uid_t uid, const char *command, const char *type)
     {
       handle = do_popen(command, type);
       if (handle < 0)
-	return NULL;
+        return NULL;
       return userfile_make_handle(USERFILE_LOCAL, handle);
     }
 
   if (!userfile_initialized)
     fatal("userfile_popen: using non-current uid but not initialized (uid=%d)",
-	  (int)uid);
+          (int)uid);
   
   if (uid != userfile_uid)
     fatal("userfile_popen: uid not current and not that of child: uid=%d",
-	  (int)uid);
+          (int)uid);
 
   userfile_packet_start(USERFILE_POPEN);
   buffer_put_string(&packet, command, strlen(command));
@@ -1100,7 +1109,7 @@ int userfile_pclose(UserFile uf)
       ret = close(uf->handle);
       ret2 = wait(NULL);
       if (ret >= 0)
-	ret = ret2;
+        ret = ret2;
       xfree(uf);
       return ret;
 
@@ -1136,19 +1145,19 @@ char *userfile_get_des_1_magic_phrase(uid_t uid)
       des_block block;
       
       memset(buf, 0, sizeof(buf));
-      sprintf(buf, "ssh.%04X", geteuid());
+      snprintf(buf, sizeof(buf), "ssh.%04X", geteuid());
       memcpy(block.c, buf, sizeof(block.c));
       if (getnetname(buf))
-	{
-	  if (key_encryptsession(buf, &block) == 0)
-	    {
-	      sprintf(buf, "%08X%08X", ntohl(block.key.high),
-		      ntohl(block.key.low));
-	      memset(block.c, 0, sizeof(block.c));
-	      phrase = xstrdup(buf);
-	      memset(buf, 0, sizeof(buf));
-	    }
-	}
+        {
+          if (key_encryptsession(buf, &block) == 0)
+            {
+              snprintf(buf, sizeof(buf), "%08X%08X", ntohl(block.key.high),
+                       ntohl(block.key.low));
+              memset(block.c, 0, sizeof(block.c));
+              phrase = xstrdup(buf);
+              memset(buf, 0, sizeof(buf));
+            }
+        }
       return phrase;
     }
   
